#  diceware -- passphrases to remember
#  Copyright (C) 2015, 2016  Uli Fouquet and contributors.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Sources of randomness.

Please register all sources as entry point in ``setup.py``. Look out for
"SystemRandomSource" for an example.

For developers of interfaces to other sources of randomness: Currently,
you can extend `diceware` random sources by registering a class, that
provides a suitable `__init__(self, options)` and a `choice(self,
sequence)` method. Optionally, you can also provide a `classmethod`
called ``update_arparse`` that will get the possibility to update the
`argparser.ArgumentParser` used by `diceware`.

The `__init__` method of your class will be called with `options`, a set
of options as parsed from the commandline. The initialization code can
use the options to determine further actions or ignore it. The
`__init__` method is also the right place to ask users for one-time
infos you need. This includes infos like the number of sides of a dice,
an API key for random.org or other infos that should not change between
generating different words (but might change from one `diceware` call
to the next).

The `choice` method then, will get a sequence of chars, strings, or
numbers and should pick one of them based on the source of randomness
intended to be utilized by your code. If further user interaction is
required, `choice` might also ask users for input or similar. Typically,
`choice` is called once for each word and once for each special char to
generate.

If you want to manage own commandline options with your plugin, you can
implement a `classmethod` called ``update_argparser(parser)`` which gets
an `argparse.ArgumentParser` instance  as argument (no pun intended).

Finally, to register the source, add some stanza in `setup.py` of your
project that looks like::

    # ...
    setup(
        # ...
        entry_points={
            # console scripts and other entry points...
            'diceware_random_sources': [
                'myrandom = mypkg.mymodule:MyRandomSource',
                'myothersrc = mypkg.mymodule:MyOtherSource',
            ],
        },
        # ...
    )
    # ...

Here the `myrandom` and `myothersrc` lines register random sources that
(if installed) `diceware` will find on startup and offer to users under
the name given. In the described case, users could do for instance::

  diceware -r myrandom

and the random source defined in the given class would be used for
generating a passphrase.

"""
import math
import sys
from random import SystemRandom


input_func = input
if sys.version[0] < "3":
    input_func = raw_input  # NOQA  # pragma: no cover


class SystemRandomSource(object):
    """A Random Source utilizing the standard Python `SystemRandom` call.

    As time of writing, SystemRandom makes use of ``/dev/urandom`` to get
    fairly useable random numbers.

    This source is registered as entry_point in setup.py under the name
    'system' in the ``diceware_random_sources`` group.

    The constructor will be called with options at beginning of a
    programme run if the user has chosen the respective source of
    random.

    The SystemRandomSource is the default source.
    """
    def __init__(self, options):
        self.options = options
        self.rnd = SystemRandom()

    def choice(self, sequence):
        """Pick one item out of `sequence`.

        The `sequence` will normally be a sequence of strings
        (wordlist), special chars, or numbers.

        Sequences can be (at least) lists, tuples and other types that
        have a `len`. Generators do not have to be supported (and are
        in fact not supported by this source).

        This method should return one item of the `sequence` picked based on
        the underlying source of randomness.

        In the long run, the choice should return each `sequence` item
        (i.e.: no items should be 'unreachable').

        It should also cope with any length > 0 of `sequence` and not
        break if a sequence is "too short" or "too long". Empty
        sequences, however, might raise exceptions.
        """
        return self.rnd.choice(sequence)


class RealDiceRandomSource(object):
    """A source of randomness working with real dice.
    """
    def __init__(self, options):
        self.options = options
        self.dice_sides = 6
        if options is not None:
            self.dice_sides = getattr(options, 'dice_sides', 6)

    def pre_check(self, num_rolls, sequence):
        """Checks performed before picking an item of a sequence.

        We make sure that `num_rolls`, the number of rolls, is in an
        acceptable range and issue an hint about the procedure.
        """
        if num_rolls == 0:
            raise(ValueError)
        if (self.dice_sides ** num_rolls) < len(sequence):
            print(
                "Warning: entropy is reduced! Using only first %s of %s "
                "words/items of your wordlist." % (
                    self.dice_sides ** num_rolls, len(sequence)
                )
            )
        print(
            "Please roll %s dice (or a single dice %s times)." % (
                num_rolls, num_rolls))
        return

    def choice(self, sequence):
        """Pick one item out of `sequence`.
        """
        if len(sequence) == 1:
            return sequence[0]  # no need to roll dice.
        num_rolls = int(math.log(len(sequence), self.dice_sides))
        if num_rolls < 1:
            # If this happens, there are less values in the sequence to
            # choose from than there are dice sides.
            # Check whether len(sequence) is a factor of dice_sides
            if self.dice_sides % len(sequence) == 0:
                num_rolls = 1
            else:
                # otherwise We will perform one extra roll and apply modulo
                num_rolls = 2
        self.pre_check(num_rolls, sequence)
        result = 0
        for i in range(num_rolls, 0, -1):
            rolled = None
            while rolled not in [
                    str(x) for x in range(1, self.dice_sides + 1)]:
                rolled = input_func(
                    "What number shows dice number %s? " % (num_rolls - i + 1))
            result += ((self.dice_sides ** (i - 1)) * (int(rolled) - 1))
            result = result % len(sequence)
        return sequence[result]



import os
import pytest
from io import StringIO
from diceware.wordlist import (
    WORDLISTS_DIR, RE_WORDLIST_NAME, RE_NUMBERED_WORDLIST_ENTRY,
    RE_VALID_WORDLIST_FILENAME, get_wordlist_path, get_wordlist_names,
    WordList,
)


@pytest.fixture(scope="function")
def wordlist(request, tmpdir):
    """A fixture that delivers a simple WordList instance.
    """
    path = tmpdir.join("mylist.txt")
    path.write("foo\nbar\n")
    w_list = WordList(str(path))
    return w_list


class TestWordlistModule(object):

    def test_re_wordlist_name(self):
        # RE_WORDLIST_NAME really works
        # valid stuff
        assert RE_WORDLIST_NAME.match('de') is not None
        assert RE_WORDLIST_NAME.match('DE') is not None
        assert RE_WORDLIST_NAME.match('vb') is not None
        assert RE_WORDLIST_NAME.match('8k') is not None
        assert RE_WORDLIST_NAME.match('original') is not None
        assert RE_WORDLIST_NAME.match('with_underscore') is not None
        assert RE_WORDLIST_NAME.match('u') is not None
        assert RE_WORDLIST_NAME.match('with-hyphen') is not None
        # invalid stuff
        assert RE_WORDLIST_NAME.match('with space') is None
        assert RE_WORDLIST_NAME.match('"with-quotation-marks"') is None
        assert RE_WORDLIST_NAME.match("'with-quotation-marks'") is None
        assert RE_WORDLIST_NAME.match('with.dot') is None
        assert RE_WORDLIST_NAME.match('with/slash') is None

    def test_re_numbered_wordlist_entry(self):
        # we accept numbers (optionally separated by single dashes) in
        # wordlist lines
        #
        # valid stuff
        assert RE_NUMBERED_WORDLIST_ENTRY.match('11111   a') is not None
        assert RE_NUMBERED_WORDLIST_ENTRY.match('1-2-2-11 1') is not None
        assert RE_NUMBERED_WORDLIST_ENTRY.match(
            '11111   a').groups() == (None, 'a', )
        assert RE_NUMBERED_WORDLIST_ENTRY.match('12211\t 1') is not None
        assert RE_NUMBERED_WORDLIST_ENTRY.match(
            '12211\t 1').groups() == (None, '1', )
        assert RE_NUMBERED_WORDLIST_ENTRY.match(
            '1-2-2-1-1\t 1-1').groups()[1] == '1-1'
        # invalid stuff
        assert RE_NUMBERED_WORDLIST_ENTRY.match('12a11 foo') is None
        assert RE_NUMBERED_WORDLIST_ENTRY.match('12--11 foo') is None
        assert RE_NUMBERED_WORDLIST_ENTRY.match('1211- foo') is None
        assert RE_NUMBERED_WORDLIST_ENTRY.match('-1211 foo') is None
        assert RE_NUMBERED_WORDLIST_ENTRY.match('foo bar') is None

    def test_re_valid_wordlist_filename(self):
        # RE_VALID_WORDLIST_FILENAME really detects filenames we allow
        # Valid filenames
        regexp = RE_VALID_WORDLIST_FILENAME
        assert regexp.match("wordlist_foo.txt") is not None
        assert regexp.match("wordlist_foo_bar.asc") is not None
        assert regexp.match("wordlist_name-withdots.txt.asc") is not None
        assert regexp.match("wordlist_en.txt") is not None
        assert regexp.match("wordlist_en_eff.txt") is not None
        assert regexp.match("wordlist_en_orig.asc") is not None
        assert regexp.match("wordlist_en_securedrop.asc") is not None
        # We can get the internal wordlist name
        assert regexp.match("wordlist_foo.txt").groups()[0] == "foo"
        assert regexp.match(
            "wordlist_foo_bar.asc").groups()[0] == "foo_bar"
        assert regexp.match(
            "wordlist_name-with.dots.txt.asc").groups()[0] == "name-with"
        # Invalid names
        assert regexp.match("wordlist-without-underscore.txt") is None
        assert regexp.match("wordlist_invalid_ch=r.txt") is None
        assert regexp.match("wordlist_without_dot_txt") is None
        assert regexp.match("nowordlist_foo.txt") is None
        assert regexp.match("wordlist_name.") is None
        assert regexp.match("wordlist_name.txt.") is None
        assert regexp.match("wordlist_name.txt..") is None
        assert regexp.match("wordlist_name.txt/..") is None
        assert regexp.match("wordlist_.txt") is None

    def test_get_wordlist_path(self, wordlists_dir):
        # we can get valid wordlist paths
        path1 = wordlists_dir.join("wordlist_foo.txt")
        path1.write("foo\n")
        path2 = wordlists_dir.join("wordlist_bar.asc")
        path2.write("bar\n")
        assert get_wordlist_path('foo') == path1
        assert get_wordlist_path('bar') == path2
        assert get_wordlist_path('zz') is None

    def test_get_wordlist_path_ignores_subdirs(self, wordlists_dir):
        # we subdirs are ignored, when looking for wordlists.
        path1 = wordlists_dir.mkdir('wordlist_subdir1.txt')
        path2 = wordlists_dir.join('wordlist_subdir2.txt')
        path2.write('foo\n')
        assert os.path.isdir(str(path1))
        assert os.path.isfile(str(path2))
        assert get_wordlist_path('subdir1') is None
        assert get_wordlist_path('subdir2') == str(path2)

    def test_get_wordlist_path_accepts_any_ext(self, wordlists_dir):
        # we cope with any filename extension, not only .txt
        path1 = wordlists_dir.join("wordlist_foo.txt")
        path1.write("foo\n")
        path2 = wordlists_dir.join("wordlist_bar.asc")
        path2.write("bar\n")
        path3 = wordlists_dir.join("wordlist_baz.txt.asc")
        path3.write("baz\n")
        assert get_wordlist_path("foo") == str(path1)
        assert get_wordlist_path("bar") == str(path2)
        assert get_wordlist_path("baz") == str(path3)
        assert get_wordlist_path("not-existing") is None

    def test_get_wordlist_path_requires_ascii(self):
        # non ASCII alphabet chars are not accepted in language specifier
        with pytest.raises(ValueError) as exc_info:
            get_wordlist_path('../../tmp')
        assert exc_info.value.args[0].startswith(
            'Not a valid wordlist name')

    def test_get_wordlist_names(self, wordlists_dir):
        # we can get wordlist names also if directory is empty.
        wlist_path = wordlists_dir.join('wordlist_my_en.txt')
        wlist_path.write("some\nirrelevant\nwords")
        assert get_wordlist_names() == ['my_en']

    def test_get_wordlist_names_files_only(self, wordlists_dir):
        # non-files are ignored when looking for wordlist names
        sub_dir = wordlists_dir.mkdir('subdir')                # a subdir
        sub_dir.join("somfile_name.txt").write("Some\ntext")   # and a file in
        assert get_wordlist_names() == []

    def test_get_wordlist_names_requires_underscore(self, wordlists_dir):
        # we only recognize wordlist files with underscore in name
        wordlists_dir.join("file-without-underscore.txt").write("a\nb\n")
        assert get_wordlist_names() == []

    def test_get_wordlist_names_requires_dot(self, wordlists_dir):
        # we only recognize wordlist files with dot in name
        wordlists_dir.join("file_without_dot-in-name").write("a\nb\n")
        assert get_wordlist_names() == []


class TestWordList(object):

    def test_create_wordlist(self, tmpdir):
        # we can create `WordList` objects.
        in_file = tmpdir.mkdir("work").join("mywordlist")
        in_file.write("foo\n")
        w_list = WordList(str(in_file))
        assert w_list is not None
        assert hasattr(w_list, "path")
        assert hasattr(w_list, "fd")
        assert hasattr(w_list, "signed")

    def test_create_opens_file(self, tmpdir):
        # if we pass-in a path, the file will be opened for reading.
        in_file = tmpdir.mkdir("work").join("mywordlist")
        in_file.write("foo\n")
        w_list = WordList(str(in_file))
        assert w_list.fd is not None

    def test_create_accepts_open_file(self, tmpdir):
        # if we pass in an open file, it will be used
        in_file = tmpdir.mkdir("work").join("mywordlist")
        in_file.write("foo\n")
        with open(str(in_file), "r") as my_open_file:
            w_list = WordList(my_open_file)
            assert w_list.fd is not None
            assert w_list.path is None

    def test_create_accepts_fd_with_broken_seek(self, argv_handler):
        # we accept files that have no working seek() (like sys.stdin)
        fd = StringIO(b"word1\nword2\n".decode("utf-8"))

        def broken_seek(num):
            raise IOError("Illegal seek")
        fd.seek = broken_seek
        w_list = WordList(fd)
        assert w_list.fd is not fd

    def test_open_file_descriptor_simple(self, tmpdir):
        # we handle simple lists from open file descriptors correctly
        in_file = tmpdir.mkdir("work").join("mywordlist")
        in_file.write("foo\nbar\n\nbaz\n")
        with open(str(in_file), "r") as my_open_file:
            w_list = WordList(my_open_file)
            result = tuple(w_list)
        assert result == ("foo", "bar", "baz")

    def test_open_file_descriptor_signed(self):
        # we can handle signed wordlists from open file descriptors
        in_path = os.path.join(
            os.path.dirname(__file__), "sample_signed_wordlist.asc")
        with open(str(in_path), "r") as my_open_file:
            w_list = WordList(my_open_file)
            result = tuple(w_list)
        assert ("foo", "bar", "-dash-at-start", "baz") == result

    def test_detect_unsigned_wordlists(self, tmpdir):
        # we can detect unsigned wordlist files.
        in_file = tmpdir.mkdir("work").join("mywordlist")
        in_file.write("foo\n")
        w_list = WordList(str(in_file))
        assert w_list.signed is False

    def test_detect_signed_wordlists(self):
        # we can detect signed wordlist files.
        in_path = os.path.join(
            os.path.dirname(__file__), "sample_signed_wordlist.asc")
        w_list = WordList(in_path)
        assert w_list.signed is True

    def test_wordlists_are_generators(self, tmpdir):
        # WordList instances act like generators.
        in_file = tmpdir.mkdir("work").join("mywordlist")
        in_file.write("foo\nbar\n")
        w_list = WordList(str(in_file))
        assert list(w_list) == ["foo", "bar"]

    def test_wordlist_from_signed_file(self):
        # we can get an iterator from signed wordlist.
        in_path = os.path.join(
            os.path.dirname(__file__), "sample_signed_wordlist.asc")
        w_list = WordList(in_path)
        assert list(w_list) == ["foo", "bar", "-dash-at-start", "baz"]

    def test_wordlist_en_8k(self):
        # we can get a list of words out of the reinhold english 8k wordlist.
        en_src = os.path.join(WORDLISTS_DIR, 'wordlist_en.txt')
        w_list = WordList(en_src)
        long_list = list(w_list)
        assert long_list[0] == "a"
        assert long_list[-1] == "@"
        assert len(long_list) == 8192

    def test_wordlist_en_securedrop(self):
        # we can get a list of words out of securedrop english 8k wordlist.
        en_src = os.path.join(WORDLISTS_DIR, 'wordlist_en_securedrop.asc')
        w_list = WordList(en_src)
        long_list = list(w_list)
        assert long_list[0] == "0"
        assert long_list[-1] == "zurich"
        assert len(long_list) == 8192

    def test_wordlist_en(self):
        # we can get a list of words out of the original diceware wordlist.
        en_src = os.path.join(WORDLISTS_DIR, 'wordlist_en_orig.asc')
        w_list = list(WordList(en_src))
        assert w_list[0] == "a"
        assert w_list[-1] == "@"
        assert len(w_list) == 7776

    def test_wordlist_en_eff(self):
        # we can get a list of words out of the EFF-maintained wordlist.
        en_src = os.path.join(WORDLISTS_DIR, 'wordlist_en_eff.txt')
        w_list = list(WordList(en_src))
        assert w_list[0] == "abacus"
        assert w_list[-1] == "zoom"
        assert len(w_list) == 7776

    def test_get_wordlist_simple(self, tmpdir):
        # simple wordlists can be created
        in_file = tmpdir.mkdir("work").join("mywordlist")
        in_file.write("a\nb\n")
        result = list(WordList(str(in_file)))
        assert ['a', 'b'] == result

    def test_get_wordlist_ignore_empty_lines(self, tmpdir):
        # we ignore empty lines in wordlists
        in_file = tmpdir.mkdir("work").join("mywordlist")
        in_file.write("\n\na\n\n")
        result = list(WordList(str(in_file)))
        assert ['a'] == result

    def test_get_signed_wordlist_handles_clearsigned_files(self):
        # we can process cryptogrphically signed files
        in_path = os.path.join(
            os.path.dirname(__file__), "sample_signed_wordlist.asc")
        with open(in_path, 'r') as fd:
            result = list(WordList(fd))
        assert ["foo", "bar", "-dash-at-start", "baz"] == result

    def test_get_signed_wordlist_handles_en_orig(self):
        # we can process the original diceware list from diceware.com
        wlist_path = os.path.join(WORDLISTS_DIR, 'wordlist_en_orig.asc')
        w_list = WordList(wlist_path)
        result = list(w_list)
        assert len(result) == 7776
        assert "a" == result[0]
        assert "@" == result[-1]

    def test_get_signed_wordlist_ignore_empty_lines(self):
        # we ignore empty lines in wordlists
        in_path = os.path.join(
            os.path.dirname(__file__), "sample_signed_wordlist.asc")
        w_list = WordList(in_path)
        result = list(w_list)
        assert '' not in result

    def test_can_get_wordlist_multiple_times(self, tmpdir):
        # we can get a wordlist several times
        in_file = tmpdir.mkdir("work").join("mywordlist")
        in_file.write("\n\na\n\n")
        w_list = WordList(str(in_file))
        list1 = list(w_list)
        list2 = list(w_list)
        assert list1 == list2

    def test_can_get_wordlist_multiple_times_from_fd(self, tmpdir):
        # we can get a wordlist several times also if it is a fd.
        in_file = tmpdir.mkdir("work").join("mywordlist")
        in_file.write("foo\nbar\n")
        with open(str(in_file), "r") as fd:
            w_list = WordList(fd)
            list1 = list(w_list)
            list2 = list(w_list)
        assert list1 == list2

    def test_is_signed_detects_signed_files(self):
        # we recognize signed wordlists
        in_path = os.path.join(
            os.path.dirname(__file__), "sample_signed_wordlist.asc")
        with open(in_path, "r") as fd:
            w_list = WordList(fd)
            w_list.fd = fd
            result = w_list.is_signed()
        assert result is True

    def test_is_signed_detects_unsigned_files(self, tmpdir):
        # we can tell if a wordlist is not signed
        in_file = tmpdir.mkdir("work").join("mywordlist")
        in_file.write("a\nb\n")
        with open(in_file.strpath, 'r') as fd:
            w_list = WordList(fd)
            w_list.fd = fd
            result = w_list.is_signed()
        assert result is False

    def test_refine_entry_strips(self, wordlist):
        # we strip() entries
        assert wordlist.refine_entry("foo") == "foo"
        assert wordlist.refine_entry(" foo \n") == "foo"
        assert wordlist.refine_entry(" foo bar \n") == "foo bar"

    def test_refine_entry_handles_numbered(self, wordlist):
        # we transform numbered lines
        assert wordlist.refine_entry("11111\tfoo") == "foo"

    def test_refine_entry_handles_dash_quotes_when_signed(
            self, wordlist):
        # we handle dash-escaped lines correctly when in signed mode
        assert wordlist.refine_entry("- foo") == "- foo"
        wordlist.signed = True
        assert wordlist.refine_entry("- foo") == "foo"

    def test_refine_entry_strips_also_dash_quoted(self, wordlist):
        # also dash-escaped lines in signed wordlistgs are stripped.
        wordlist.signed = True
        assert wordlist.refine_entry("- \tfoo\n") == "foo"

    def test_refine_entry_strips_also_numbered(self, wordlist):
        # also numbered entries are stripped
        assert wordlist.refine_entry("11111 \t foo\n") == "foo"

    def test_refine_entry_can_handle_all_at_once(self, wordlist):
        # we can do all the things above at once and in right order.
        wordlist.signed = True
        assert wordlist.refine_entry("- 11111 foo  \n") == "foo"



#  diceware -- passphrases to remember
#  Copyright (C) 2015, 2016  Uli Fouquet and contributors.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""wordlist.py -- special handling of wordlists.
"""
import os
import re
import tempfile

#: Maximum in-memory file size in bytes (20 MB).
#:
#: This value is used when creating temporary files replacing
#: unseekable input streams. If an input file is larger, we write to
#: disk.
MAX_IN_MEM_SIZE = 20 * 1024 * 1024

#: The directory in which wordlists are stored
WORDLISTS_DIR = os.path.abspath(
    os.path.join(os.path.dirname(__file__), 'wordlists'))

#: A regular expression matching allowed wordlist names. We
#: allow names that cannot easily mess up filesystems.
RE_WORDLIST_NAME = re.compile('^[\w-]+$')

#: A regular expression matching numbered entries in wordlists.
RE_NUMBERED_WORDLIST_ENTRY = re.compile('^[0-9]+(\-[0-9]+)*\s+([^\s]+)$')

#: A regular expression describing valid wordlist file names.
RE_VALID_WORDLIST_FILENAME = re.compile(
    '^wordlist_([\w-]+)\.[\w][\w\.]+[\w]+$')


def get_wordlist_names():
    """Get a all names of wordlists stored locally.
    """
    result = []
    filenames = os.listdir(WORDLISTS_DIR)
    for filename in filenames:
        if not os.path.isfile(os.path.join(WORDLISTS_DIR, filename)):
            continue
        match = RE_VALID_WORDLIST_FILENAME.match(filename)
        if not match:
            continue
        result.append(match.groups()[0])
    return sorted(result)


def get_wordlist_path(name):
    """Get path to a wordlist file for a wordlist named `name`.

    The `name` string must not contain special chars beside ``-``,
    ``_``, regular chars ``A-Z`` (upper or lower case) or
    numbers. Invalid names raise a ValueError.

    If a path with the given name (names are not filenames here) does
    not exist, `None` is returned.
    """
    if not RE_WORDLIST_NAME.match(name):
        raise ValueError("Not a valid wordlist name: %s" % name)
    for filename in os.listdir(WORDLISTS_DIR):
        if not os.path.isfile(os.path.join(WORDLISTS_DIR, filename)):
            continue
        match = RE_VALID_WORDLIST_FILENAME.match(filename)
        if match and match.groups()[0] == name:
            return os.path.join(WORDLISTS_DIR, filename)


class WordList(object):
    """A word list contains words for building passphrases.

    `path_or_filelike` is the path of the wordlist file or an already
    opened file. Opened files must be open for reading, of course. We
    expect filelike objects to support at least `read()`.

    If a file-like object does not support `seek()` (like `sys.stdin`),
    we create a temporary, seekable copy of the input stream. The copy
    is written to disk only, if it is larger than
    `MAX_IN_MEM_SIZE`. Otherwise the wordlist is kept in memory.

    Please note that open file descriptors are not closed after reading.

    Wordlist files are expected to contain words, one word per
    line. Empty lines are ignored, also whitespaces before or trailing
    a line are stripped. If a "word" contains inner whitespaces, then
    these are preserved.

    The input file can be a signed wordlist. Signed wordlists are
    expected to be ordinary lists of words but with ASCII armored
    signatures (as described in RFC 4880).

    In case of signed wordlists the signature headers/footers are
    stripped and the contained list of words is read.

    WordList are generators. That means, that you can retrieve the
    words of a wordlist by iterating over an instance of `WordList`.

    """
    def __init__(self, path_or_filelike=None):
        self.path = None
        if not hasattr(path_or_filelike, 'read'):
            # got a path, not a filelike object
            self.path = path_or_filelike
            self.fd = open(self.path, "r")
        else:
            self.fd = path_or_filelike
            try:
                self.fd.seek(0)
            except IOError:
                # the given filelike does not support seek(). Create an own.
                self.fd = tempfile.SpooledTemporaryFile(
                    max_size=MAX_IN_MEM_SIZE, mode="w+")
                self.fd.write(path_or_filelike.read())
                self.fd.seek(0)
        self.signed = self.is_signed()

    def __iter__(self):
        self.fd.seek(0)
        if self.signed:
            while self.fd.readline().strip():
                # wait for first empty line
                pass
        for line in self.fd:
            line = self.refine_entry(line)
            if not line:
                continue
            elif self.signed and line == '-----BEGIN PGP SIGNATURE-----':
                break
            yield line

    def is_signed(self):
        """check, whether this file is cryptographically signed.

        This operation is expensive and resets the file descriptor to
        the beginning of file.
        """
        self.fd.seek(0)
        line1 = self.fd.readline()
        self.fd.seek(0)
        if line1.rstrip() == "-----BEGIN PGP SIGNED MESSAGE-----":
            return True
        return False

    def refine_entry(self, entry):
        """Apply modifications to form a proper wordlist entry.

        Refining means: strip() `entry` remove escape-dashes (if this is
        a signed wordlist) and extract the term if it is preceded by
        numbers.
        """
        if self.signed and entry.startswith('- '):
            entry = entry[2:]
        entry = entry.strip()
        match = RE_NUMBERED_WORDLIST_ENTRY.match(entry)
        if match:
            entry = match.groups()[1]
        return entry



import pytest
import sys


class InputMock(object):
    """A replacement for input() or raw_input() respectively.

    This mock, when called, mimics input() behaviour, outputs a prompt,
    etc., but does not wait for real key strokes. Instead it returns the
    next value from `fake_input_values` given on initialization:

       >>> faked_input = InputMock(["val1", "val2", "1"])
       >>> faked_input("Give a value: ")
       Give a value: val1
       'val1'

       >>> faked_input("And another value: ")
       And another value: val2
       'val2'

       >>> faked_input()
       1
       '1'

    To be used with the `monkeypatch` pytest fixture, to replace
    `diceware.random_sources.input_func`.
    """
    fake_input_values = []

    def __init__(self, fake_input_values=[]):
        self.fake_input_values = fake_input_values
        self.fake_input_values.reverse()

    def __call__(self, prompt=''):
        curr_value = self.fake_input_values.pop()
        print("%s%s" % (prompt, curr_value))
        return curr_value


@pytest.fixture(scope="function")
def argv_handler(request):
    """This fixture restores sys.argv and sys.stdin after tests.
    """
    _argv_stored = sys.argv
    _stdin_stored = sys.stdin

    def teardown():
        sys.argv = _argv_stored
        sys.stdin = _stdin_stored
    request.addfinalizer(teardown)


@pytest.fixture(scope="function")
def wordlists_dir(request, monkeypatch, tmpdir):
    """This fixture provides a temporary wordlist dir.
    """
    monkeypatch.setattr("diceware.wordlist.WORDLISTS_DIR", str(tmpdir))
    return tmpdir


@pytest.fixture(scope="function")
def home_dir(request, monkeypatch, tmpdir):
    """This fixture provides a temporary user home.
    """
    tmpdir.mkdir("home")
    monkeypatch.setenv("HOME", str(tmpdir / "home"))
    return tmpdir / "home"


@pytest.fixture(autouse=True)
def change_home(monkeypatch, tmpdir):
    """Set $HOME to some tempdir.

    This is an autouse fixture.

    If the user running tests has an own .diceware.ini in his home, then
    this will influence tests. Therefore we set the user home to some
    empty dir while tests are running.
    """
    monkeypatch.setenv("HOME", str(tmpdir))
    return tmpdir



import os
from diceware.config import (
    OPTIONS_DEFAULTS, valid_locations, get_configparser, get_config_dict,
    configparser,
    )


class TestConfigModule(object):
    # tests for diceware.config

    def test_defaults(self):
        # there is a set of defaults for options available
        assert OPTIONS_DEFAULTS is not None

    def test_valid_locations(self, home_dir):
        # we look for config files in user home and local dir
        assert valid_locations() == [
            str(home_dir / ".diceware.ini")
            ]

    def test_get_configparser(self, tmpdir):
        # we can parse simple configs
        conf_path = tmpdir / "sample.ini"
        conf_path.write("\n".join(["[diceware]", "num=1", ""]))
        found, config = get_configparser([str(conf_path), ])
        assert found == [str(conf_path)]

    def test_get_configparser_empty_list(self):
        # we cope with empty config file lists
        found, config = get_configparser([])
        assert found == []

    def test_get_configparser_no_list(self, home_dir):
        # we cope with no list at all
        found, config = get_configparser()
        assert found == []

    def test_get_configparser_default_path(self, home_dir):
        # a config file in $HOME is looked up by default
        config_file = home_dir / ".diceware.ini"
        config_file.write("\n".join(["[diceware]", "num = 3", ""]))
        found, config = get_configparser()
        assert found == [str(config_file)]


class TestGetConfigDict(object):
    # tests for get_config_dict()

    def test_get_config_dict_no_config_file(self, home_dir):
        # we get config values even without a config file.
        conf_dict = get_config_dict()
        assert conf_dict == OPTIONS_DEFAULTS
        assert conf_dict is not OPTIONS_DEFAULTS

    def test_get_config_dict_no_diceware_section(self, home_dir):
        # we cope with config files, if they do not contain a diceware config
        config_file = home_dir / ".diceware.ini"
        config_file.write("\n".join(["[not-diceware]", "num = 3", ""]))
        conf_dict = get_config_dict()
        assert conf_dict == OPTIONS_DEFAULTS

    def test_get_config_dict(self, home_dir):
        # we can get config values from files as a dict.
        config_file = home_dir / ".diceware.ini"
        config_file.write("\n".join(["[diceware]", "num = 3", ""]))
        conf_dict = get_config_dict()
        assert len(conf_dict) == len(OPTIONS_DEFAULTS)
        assert conf_dict != OPTIONS_DEFAULTS

    def test_get_config_dict_arg_path_list(self, home_dir):
        # we can give the paths searched.
        config_file_default = home_dir / ".diceware.ini"
        config_file_default.write("[diceware]\nnum=4\n")
        config_file_custom = home_dir / "some-new-file"
        config_file_custom.write("[diceware]\nnum=42\n")
        conf_dict = get_config_dict(
            path_list=[str(config_file_custom), ])
        assert conf_dict['num'] == 42

    def test_get_config_dict_arg_defaults_dict(self, home_dir):
        # we can change the dict of defaults used.
        custom_defaults = dict(num=42)
        conf_dict = get_config_dict(defaults_dict=custom_defaults)
        assert conf_dict == dict(num=42)
        assert conf_dict is not custom_defaults

    def test_get_config_dict_arg_section(self, home_dir):
        # we can set the section name to look for in config files.
        config_file = home_dir / ".diceware.ini"
        config_file.write("[diceware]\nnum=4\n[foo]\nnum=5\n[bar]\nnum=6\n")
        conf_dict = get_config_dict(section='foo')
        assert conf_dict['num'] == 5

    def test_get_config_dict_int(self, home_dir):
        # integer values are interpolated correctly
        config_file = home_dir / ".diceware.ini"
        config_file.write("\n".join(["[diceware]", "num=3", ""]))
        conf_dict = get_config_dict()
        assert "num" in conf_dict.keys()
        assert conf_dict["num"] == 3

    def test_get_config_dict_bool(self, home_dir):
        # boolean values are interpolated correctly
        config_file = home_dir / ".diceware.ini"
        config_file.write("\n".join(["[diceware]", "caps = Off", ""]))
        conf_dict = get_config_dict()
        assert "caps" in conf_dict.keys()
        assert conf_dict["caps"] is False
        config_file.write("\n".join(["[diceware]", "caps = On", ""]))
        assert get_config_dict()["caps"] is True

    def test_get_config_dict_ignore_irrelevant(self, home_dir):
        # values that have no default are ignored
        config_file = home_dir / ".diceware.ini"
        config_file.write("\n".join(["[diceware]", "foo = bar", ""]))
        conf_dict = get_config_dict()
        assert "foo" not in conf_dict.keys()

    def test_get_config_dict_string(self, home_dir):
        # string values are interpolated correctly
        config_file = home_dir / ".diceware.ini"
        config_file.write("\n".join(["[diceware]", "delimiter=!", ""]))
        conf_dict = get_config_dict()
        assert conf_dict["delimiter"] == "!"

    def test_get_config_dict_string_empty(self, home_dir):
        # we can set empty string values
        config_file = home_dir / ".diceware.ini"
        config_file.write("\n".join(["[diceware]", "delimiter=", ""]))
        conf_dict = get_config_dict()
        assert conf_dict["delimiter"] == ""

    def test_get_config_dict_string_quotes_stripped(self, home_dir):
        # preceding/trailing quotes are stripped from string values
        config_file = home_dir / ".diceware.ini"
        for string_in_conf, expected_parsed in (
                ('" "', " "), ("' '", " "), ('"\t\t"', "\t\t")):
            config_file.write("[diceware]\ndelimiter=%s \n" % string_in_conf)
            conf_dict = get_config_dict()
            assert conf_dict["delimiter"] == expected_parsed


class TestSampleIni(object):
    # test local sample ini file

    def test_complete_options_set(self, home_dir):
        # make sure the set of options in sample file is complete
        sample_path = os.path.join(
            os.path.dirname(__file__), 'sample_dot_diceware.ini')
        parser = configparser.SafeConfigParser()
        found = parser.read([sample_path, ])
        assert sample_path in found
        assert parser.has_section('diceware')
        for key, val in OPTIONS_DEFAULTS.items():
            # make sure option keywords are contained.
            assert parser.has_option('diceware', key)

    def test_no_invalid_options(self, home_dir):
        # ensure we have no obsolete/unused options in sample
        sample_path = os.path.join(
            os.path.dirname(__file__), 'sample_dot_diceware.ini')
        parser = configparser.SafeConfigParser()
        parser.read([sample_path, ])
        for option in parser.options('diceware'):
            assert option in OPTIONS_DEFAULTS.keys()



#  diceware -- passphrases to remember
#  Copyright (C) 2015, 2016  Uli Fouquet
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""diceware -- rememberable passphrases
"""
import argparse
import pkg_resources
import sys
from random import SystemRandom
from diceware.config import get_config_dict
from diceware.wordlist import (
    WordList, get_wordlist_path, WORDLISTS_DIR, get_wordlist_names,
    )

__version__ = pkg_resources.get_distribution('diceware').version

#: Special chars inserted on demand
SPECIAL_CHARS = r"~!#$%^&*()-=+[]\{}:;" + r'"' + r"'<>?/0123456789"


GPL_TEXT = (
    """
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    """
    )


def print_version():
    """Output current version and other infos.
    """
    print("diceware %s" % __version__)
    print("Copyright (C) 2015, 2016 Uli Fouquet")
    print("diceware is based on suggestions of Arnold G. Reinhold.")
    print("See http://diceware.com for details.")
    print("'Diceware' is a trademark of Arnold G. Reinhold.")
    print(GPL_TEXT)


def get_random_sources():
    """Get a dictionary of all entry points called diceware_random_source.

    Returns a dictionary with names mapped to callables registered as
    `entry_point`s for the ``diceware_randomsource`` group.

    Callables should accept `options` when called and return something
    that provides a `choice(sequence)` method that works like the
    respective method in the standard Python lib `random` module.
    """
    result = dict()
    for entry_point in pkg_resources.iter_entry_points(
            group="diceware_random_sources"):
        result.update({entry_point.name: entry_point.load()})
    return result


def handle_options(args):
    """Handle commandline options.
    """
    plugins = get_random_sources()
    random_sources = plugins.keys()
    wordlist_names = get_wordlist_names()
    defaults = get_config_dict()
    parser = argparse.ArgumentParser(
        description="Create a passphrase",
        epilog="Wordlists are stored in %s" % WORDLISTS_DIR
        )
    parser.add_argument(
        '-n', '--num', default=6, type=int,
        help='number of words to concatenate. Default: 6')
    cap_group = parser.add_mutually_exclusive_group()
    cap_group.add_argument(
        '-c', '--caps', action='store_true',
        help='Capitalize words. This is the default.')
    cap_group.add_argument(
        '--no-caps', action='store_false', dest='caps',
        help='Turn off capitalization.')
    parser.add_argument(
        '-s', '--specials', default=0, type=int, metavar='NUM',
        help="Insert NUM special chars into generated word.")
    parser.add_argument(
        '-d', '--delimiter', default='',
        help="Separate words by DELIMITER. Empty string by default.")
    parser.add_argument(
        '-r', '--randomsource', default='system', choices=random_sources,
        metavar="SOURCE",
        help=(
            "Get randomness from this source. Possible values: `%s'. "
            "Default: system" % "', `".join(sorted(random_sources))))
    parser.add_argument(
        '-w', '--wordlist', default='en_securedrop', choices=wordlist_names,
        metavar="NAME",
        help=(
            "Use words from this wordlist. Possible values: `%s'. "
            "Wordlists are stored in the folder displayed below. "
            "Default: en_securedrop" % "', `".join(wordlist_names)))
    realdice_group = parser.add_argument_group(
        "Arguments related to `realdice' randomsource",
        )
    realdice_group.add_argument(
            '--dice-sides', default=6, type=int, metavar="N",
            help='Number of sides of dice. Default: 6'
        )
    parser.add_argument(
        'infile', nargs='?', metavar='INFILE', default=None,
        type=argparse.FileType('r'),
        help="Input wordlist. `-' will read from stdin.",
        )
    parser.add_argument(
        '-v', '--verbose', action='count',
        help='Be verbose. Use several times for increased verbosity.')
    parser.add_argument(
        '--version', action='store_true',
        help='output version information and exit.',
        )
    for plugin in plugins.values():
        if hasattr(plugin, "update_argparser"):
            parser = plugin.update_argparser(parser)
    parser.set_defaults(**defaults)
    args = parser.parse_args(args)
    return args


def insert_special_char(word, specials=SPECIAL_CHARS, rnd=None):
    """Insert a char out of `specials` into `word`.

    `rnd`, if passed in, will be used as a (pseudo) random number
    generator. We use `.choice()` only.

    Returns the modified word.
    """
    if rnd is None:
        rnd = SystemRandom()
    char_list = list(word)
    char_list[rnd.choice(range(len(char_list)))] = rnd.choice(specials)
    return ''.join(char_list)


def get_passphrase(options=None):
    """Get a diceware passphrase.

    `options` is a set of arguments as provided by
    `argparse.OptionParser.parse_args()`.

    The passphrase returned will contain `options.num` words deliimted by
    `options.delimiter` and `options.specials` special chars.

    For the passphrase generation we will use the random source
    registered under the name `options.randomsource` (something like
    "system" or "dice").

    If `options.caps` is ``True``, all words will be caps.

    If `options.infile`, a file descriptor, is given, it will be used
    instead of a 'built-in' wordlist. `options.infile` must be open for
    reading.
    """
    if options is None:
        options = handle_options(args=[])
    if options.infile is None:
        options.infile = open(get_wordlist_path(options.wordlist), 'r')
    word_list = WordList(options.infile)
    rnd_source = get_random_sources()[options.randomsource]
    rnd = rnd_source(options)
    words = [rnd.choice(list(word_list)) for x in range(options.num)]
    if options.caps:
        words = [x.capitalize() for x in words]
    result = options.delimiter.join(words)
    for _ in range(options.specials):
        result = insert_special_char(result, rnd=rnd)
    return result


def main(args=None):
    """Main programme.

    Called when `diceware` script is called.

    `args` is a list of command line arguments to process. If no such
    args are given, we use `sys.argv`.
    """
    if args is None:
        args = sys.argv[1:]
    options = handle_options(args)
    if options.version:
        print_version()
        raise SystemExit(0)
    print(get_passphrase(options))



#  diceware -- passphrases to remember
#  Copyright (C) 2016  Uli Fouquet and contributors
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""logging -- output status and other data.

The `logger` provided in this module is meant to be used by other
components for messages to users.

It is named `"ulif.openoffice"` and can, as a singleton, be retrieved by
calling standard lib `logging.getLogger("ulif.diceware")`.

By default it provides a `logging.NullHandler` as libraries normally
do. Other components might add other handlers.

"""
import logging
try:
    from logging import NullHandler
except ImportError:  # NOQA  # pragma: no cover
    class NullHandler(object):
        """Replacement for `logging.NullHandler` from py3.x standard lib.
        """
        level = None

        def emit(self, record):
            pass

        def handle(self, record):
            pass

        def createLock(self):
            pass


#: Logger that can be used for all diceware related messages.
logger = logging.getLogger("ulif.diceware")
logger.addHandler(NullHandler())


def configure(verbosity=None):
    """Configure global duceware logger.

    `verbosity` sets the diceware logger verbosity. ``0`` enables info
    mode, while all numbes > 2 enable debug mode.
    """
    if verbosity is None:
        return
    logger.setLevel(logging.CRITICAL)
    if verbosity == 1:
        logger.setLevel(logging.INFO)
    elif verbosity > 1:
        logger.setLevel(logging.DEBUG)
    logger.addHandler(logging.StreamHandler())
    logger.debug("Verbose logging enabled")



import logging
from diceware.logger import logger, configure


def test_logger_exists():
    # the logger does exist
    assert logger is not None


def test_logger_has_handler():
    # the logger has at least one handler
    assert len(logger.handlers) > 0


def test_get_logger_by_name():
    # we can get a logger directly from std lib
    my_logger = logging.getLogger("ulif.diceware")
    assert len(my_logger.handlers) > 0


def test_configure():
    # we can configure the logger.
    my_logger = logging.getLogger("ulif.diceware")
    configure(None)
    assert my_logger.level == logging.NOTSET
    configure(0)
    assert my_logger.level == logging.CRITICAL
    configure(1)
    assert my_logger.level == logging.INFO
    configure(2)
    assert my_logger.level == logging.DEBUG



from __future__ import unicode_literals
import datetime
import os
import pytest
import re
import sys
from io import StringIO
from diceware import (
    WORDLISTS_DIR, SPECIAL_CHARS, insert_special_char, get_passphrase,
    handle_options, main, __version__, print_version, get_random_sources,
    get_wordlist_names
    )


class FakeRandom(object):
    # a very, very bad random generator.
    # Very handy for tests, though :-)

    nums_to_draw = [0] * 100

    def choice(self, elems):
        num, self.nums_to_draw = self.nums_to_draw[0], self.nums_to_draw[1:]
        return elems[num]


class TestHandleOptions(object):
    # tests for diceware.handle_options

    def test_handle_options(self):
        # we can get help
        with pytest.raises(SystemExit) as exc_info:
            handle_options(['--help'])
        assert exc_info.value.code == 0

    def test_handle_options_defaults(self):
        # defaults are correctly set
        options = handle_options([])
        assert options.num == 6
        assert options.caps is True
        assert options.specials == 0
        assert options.infile is None
        assert options.version is False
        assert options.delimiter == ""
        assert options.randomsource == "system"
        assert options.wordlist == "en_securedrop"
        assert options.verbose == 0

    def test_handle_options_infile(self, tmpdir):
        # we can give an infile
        tmpdir.chdir()
        with open('mywords', 'w') as fd:
            fd.write('one\ntwo\n')
        options = handle_options(['mywords', ])
        assert options.infile is not None
        assert options.infile.read() == 'one\ntwo\n'

    def test_handle_options_version(self):
        # we can ask for version infos
        options = handle_options(['--version', ])
        assert options.version is True

    def test_handle_options_caps(self):
        # we can set a flag to tell use of caps
        options = handle_options([])
        assert options.caps is True  # default
        options = handle_options(['-c', ])
        assert options.caps is True
        options = handle_options(['--caps', ])
        assert options.caps is True
        options = handle_options(['--no-caps', ])
        assert options.caps is False

    def test_handle_options_caps_conflicting_raises_exc(self):
        # conflicting caps-settings raise an exception
        with pytest.raises(SystemExit):
            handle_options(['--caps', '--no-caps'])
        with pytest.raises(SystemExit):
            handle_options(['--no-caps', '--caps'])

    def test_handle_options_delimiter(self):
        # we can set delimiter
        options = handle_options(['-d', ' '])
        assert options.delimiter == ' '
        options = handle_options(['--delimiter', ' '])
        assert options.delimiter == ' '
        options = handle_options(['-d', 'WOW'])
        assert options.delimiter == 'WOW'

    def test_handle_options_randomsource(self):
        # we can choose the source of randomness
        source_names = get_random_sources().keys()
        for name in source_names:
            options = handle_options(['-r', name])
            assert options.randomsource == name
            options = handle_options(['--randomsource', name])
            assert options.randomsource == name

    def test_handle_options_randomsource_rejects_invalid(self, capsys):
        # we can not choose illegal values for source of randomness
        with pytest.raises(SystemExit):
            handle_options(['-r', 'not-a-valid-source-name'])
        out, err = capsys.readouterr()
        assert out == ''
        assert "invalid choice" in err

    def test_handle_options_verbose(self):
        # we can set verbosity level.
        options = handle_options([])
        assert options.verbose == 0
        options = handle_options(['-v', ])
        assert options.verbose == 1
        options = handle_options(['-vv', ])
        assert options.verbose == 2
        options = handle_options(['--verbose', ])
        assert options.verbose == 1
        options = handle_options(['--verbose', '--verbose', ])
        assert options.verbose == 2

    def test_handle_options_wordlist(self, capsys):
        # we can pick a wordlist
        wordlist_names = get_wordlist_names()
        for name in wordlist_names:
            options = handle_options(['-w', name])
            assert options.wordlist == name
            options = handle_options(['--wordlist', name])
            assert options.wordlist == name

    def test_handle_options_wordlist_rejects_invalid(self, capsys):
        # we can choose only existing wordlists
        with pytest.raises(SystemExit):
            handle_options(['-w', 'not-a-valid-wordlist-name'])
        out, err = capsys.readouterr()
        assert out == ''
        assert "invalid choice" in err

    def test_handle_options_dice_sides(self):
        # we can set the number of dice sides.
        options = handle_options([])
        assert options.dice_sides == 6
        options = handle_options(['--dice-sides', '21'])
        assert options.dice_sides == 21

    def test_handle_options_considers_configfile(self, home_dir):
        # defaults from a local configfile are respected
        config_file = home_dir / ".diceware.ini"
        config_file.write("\n".join(
            ["[diceware]",
             "num = 3",
             "caps = off",
             "delimiter = my-delim",
             ""]))
        options = handle_options([])
        assert options.num == 3
        assert options.delimiter == "my-delim"
        assert options.caps is False

    def test_handle_options_allows_plugins_updating(self, monkeypatch):
        # we allow plugins to update our argparser, before being used
        import diceware

        class FakePlugin(object):
            @classmethod
            def update_argparser(cls, parser):
                parser.add_argument('--foo', default=2, type=int)
                return parser

        monkeypatch.setattr(
            diceware, 'get_random_sources', lambda: dict(foo=FakePlugin))
        options = handle_options([])
        assert options.foo == 2


class TestDicewareModule(object):

    def test_get_random_sources(self):
        # we can get a dict of random sources registered as entry_points.
        sources_dict = get_random_sources()
        assert isinstance(sources_dict, dict)
        assert len(sources_dict) > 0
        assert 'system' in sources_dict
        assert isinstance(sources_dict['system'], type)

    def test_insert_special_char(self):
        # we can insert special chars in words.
        fake_rnd = FakeRandom()
        result1 = insert_special_char('foo', specials='bar', rnd=fake_rnd)
        assert result1 == 'boo'
        fake_rnd.nums_to_draw = [1, 1]
        result2 = insert_special_char('foo', specials='bar', rnd=fake_rnd)
        assert result2 == 'fao'
        fake_rnd.nums_to_draw = [2, 2]
        result3 = insert_special_char('foo', specials='bar', rnd=fake_rnd)
        assert result3 == 'for'
        fake_rnd.nums_to_draw = [0, 0]
        result4 = insert_special_char('foo', rnd=fake_rnd)
        assert result4 == '~oo'

    def test_insert_special_char_defaults(self):
        # defaults are respected
        expected_matrix = []
        for i in range(3):
            for j in range(len(SPECIAL_CHARS)):
                word = list('foo')
                word[i] = SPECIAL_CHARS[j]
                expected_matrix.append(''.join(word))
        for x in range(100):
            assert insert_special_char('foo') in expected_matrix

    def test_special_chars_do_not_quote(self):
        # backslashes in SPECIAL_CHAR do not hide away chars
        assert len(SPECIAL_CHARS) == 36

    def test_get_passphrase(self):
        # we can get passphrases
        r1 = get_passphrase()
        r2 = get_passphrase()
        assert r1 != r2

    def test_get_passphrase_capitals(self):
        # by default a passphrase contains upper case chars
        phrase = get_passphrase()
        assert phrase.lower() != phrase

    def test_get_passphrase_no_capitals(self):
        # we can turn capitals off
        options = handle_options(args=[])
        options.caps = False
        phrase = get_passphrase(options)
        assert phrase.lower() == phrase

    def test_get_passphrase_specialchars(self):
        # we can request special chars in passphrases
        options = handle_options(args=[])
        options.specials = 2
        phrase = get_passphrase(options)
        specials = [x for x in phrase if x in SPECIAL_CHARS]
        # the 2nd special char position might be equal to 1st.
        assert len(specials) > 0

    def test_get_passphrase_delimiters(self):
        # we can set separators
        options = handle_options(args=[])
        options.delimiter = " "
        phrase = get_passphrase(options)
        assert " " in phrase

    def test_get_passphrase_wordlist_fd(self):
        #  we can pass in an own wordlist
        options = handle_options(args=[])
        options.infile = StringIO("word1\n")
        phrase = get_passphrase(options)
        assert "Word1" in phrase

    def test_print_version(self, capsys):
        # we can print version infos
        print_version()
        out, err = capsys.readouterr()
        assert err == ''
        assert __version__ in out

    def test_print_version_current_year(self, capsys):
        # in version infos we display the current year
        print_version()
        pattern = ".*\(C\) (20[0-9]{2}, )*%s.*" % (
            datetime.datetime.now().year)
        out, err = capsys.readouterr()
        assert re.match(pattern, out, re.M + re.S) is not None

    def test_main(self, capsys):
        # we can get a passphrase
        main([])  # call with default options in place
        out, err = capsys.readouterr()
        assert err == ''               # we got no errors
        assert out[-1] == '\n'         # output ends with liebreak
        assert not ('\n' in out[:-1])  # we get one line
        assert len(out) > 5            # we get at least some chars

    def test_main_help(self, argv_handler, capsys):
        # we can get help
        sys.argv = ['diceware', '--help']
        with pytest.raises(SystemExit) as exc_info:
            main()
        assert exc_info.value.code == 0
        out, err = capsys.readouterr()
        expected_path = os.path.join(
            os.path.dirname(__file__), 'exp_help_output.txt')
        with open(expected_path, 'r') as fd:
            expected_output = fd.read()
        out = out.replace(WORDLISTS_DIR, "<WORDLISTS-DIR>")
        out = out.replace("\n<WORDLISTS-DIR>", " <WORDLISTS-DIR>")
        assert out == expected_output

    def test_main_version(self, argv_handler, capsys):
        # we can get version infos.
        sys.argv = ['diceware', '--version']
        with pytest.raises(SystemExit) as exc_info:
            main()
        assert exc_info.value.code == 0
        out, err = capsys.readouterr()
        assert __version__ in out

    def test_main_argv(self, argv_handler):
        # main() handles sys.argv if nothing is provided
        sys.argv = ['diceware', '--help']
        with pytest.raises(SystemExit) as exc_info:
            main()
        assert exc_info.value.code == 0

    def test_main_infile(self, argv_handler, tmpdir, capsys):
        # main() reads custom wordlist if provided
        custom_path = tmpdir.join('mywordlist.txt')
        custom_path.write('mysingleword\n')
        tmpdir.chdir()
        main(['-n', '1', 'mywordlist.txt', ])
        out, err = capsys.readouterr()
        assert out == 'Mysingleword\n'

    def test_main_infile_stdin(self, argv_handler, capsys):
        # main() also accepts input from stdin
        sys.stdin = StringIO("word1\n")
        sys.argv = ['diceware', '-n', '2', '-']
        main()
        out, err = capsys.readouterr()
        assert out == 'Word1Word1\n'

    def test_main_delimiters(self, argv_handler, capsys):
        # delimiters are respected on calls to main
        sys.stdin = StringIO("word1\n")
        sys.argv = ['diceware', '-n', '2', '-d', 'DELIM', '-']
        main()
        out, err = capsys.readouterr()
        assert out == 'Word1DELIMWord1\n'

    def test_main_specialchars(self, argv_handler, capsys):
        # number of specialchars is respected in calls to main.
        sys.stdin = StringIO("word1\n")
        sys.argv = ['diceware', '-n', '1', '-s', '1', '-']
        main()
        out, err = capsys.readouterr()
        specials = [x for x in out if x in SPECIAL_CHARS]
        assert len(specials) > 0

    def test_main_wordlist(self, argv_handler, capsys, wordlists_dir):
        # we can pick the wordlist we prefer
        wordlists_dir.join('wordlist_foo.txt').write("foo\n")
        wordlists_dir.join('wordlist_bar.asc').write("bar\n")
        sys.argv = ['diceware', '-w', 'foo']
        main()
        out, err = capsys.readouterr()
        assert out == 'FooFooFooFooFooFoo\n'



from __future__ import unicode_literals
import pkg_resources
import pytest
import sys
import argparse
from conftest import InputMock
from io import StringIO
from diceware import main
from diceware.random_sources import (
    SystemRandomSource, RealDiceRandomSource,
    )


class TestSystemRandomSource(object):

    def test_options_are_stored(self):
        # options passed-in are stored with SystemRandomStource instances
        options = "fake_options"
        src = SystemRandomSource(options)
        assert src.options is options

    def test_has_choice_method(self):
        # SystemRandomInstances provide a choice method
        src = SystemRandomSource(None)
        assert hasattr(src, 'choice')

    def test_registered_as_system(self):
        # The SystemRandomInstance is registered as entry point with
        # name 'system' in group 'diceware_random_sources'
        sources_dict = dict()
        for entry_point in pkg_resources.iter_entry_points(
                group="diceware_random_sources"):
            sources_dict.update({entry_point.name: entry_point.load()})
        assert 'system' in sources_dict
        assert sources_dict['system'] == SystemRandomSource

    def test_choice_accepts_lists_of_numbers(self):
        # the choice() method accepts lists of numbers
        src = SystemRandomSource(None)
        assert src.choice([1, 2, 3]) in [1, 2, 3]

    def test_choice_accepts_tuples_of_numbers(self):
        # the choce() method accepts tuples of numbers
        src = SystemRandomSource(None)
        assert src.choice((1, 2, 3), ) in [1, 2, 3]

    def test_choice_accepts_list_of_chars(self):
        # the choice() method accepts lists of chars
        src = SystemRandomSource(None)
        assert src.choice(['a', 'b', 'c']) in ['a', 'b', 'c']

    def test_choice_accepts_list_of_strings(self):
        # the choice() method accepts lists of strings
        src = SystemRandomSource(None)
        assert src.choice(['foo', 'bar', 'baz']) in ['foo', 'bar', 'baz']

    def test_choice_picks_all_items(self):
        # make sure all items of a sequence are picked (in the long run)
        sequence = [1, 2, 3, 4]
        picked = set()
        num = 10 ** 3
        src = SystemRandomSource(None)
        while num:
            picked.add(src.choice(sequence))
            if len(picked) == len(sequence):
                break
            num -= 1
        assert num > 0


class TestRealDiceRandomSource(object):

    @classmethod
    def fake_input_values(cls, values, patch):
        input_mock = InputMock(values)
        patch.setattr(
            # function to replace, replacement
            "diceware.random_sources.input_func", input_mock)
        return input_mock

    def test_raw_input_patch_works(self, monkeypatch, capsys):
        # make sure our fake input works. We try to fake input ('foo',
        # 'bar') and make sure that output is captured.
        # This test is just a hint, how input could be faked in real tests.
        # It can (and should) be removed if not needed any more.
        self.fake_input_values(["foo", "bar"], monkeypatch)
        # late import, because we need the patched version
        from diceware.random_sources import input_func
        result1 = input_func("Enter some values: ")
        assert result1 == "foo"
        result2 = input_func("Enter more values: ")
        assert result2 == "bar"
        out, err = capsys.readouterr()             # captured stdout/stderr
        assert out == "Enter some values: foo\nEnter more values: bar\n"

    def test_options_are_stored(self):
        # options passed-in are stored with RealDiceRandomSource instances
        options = "fake_check"
        src = RealDiceRandomSource(options)
        assert src.options is options

    def test_has_choice_method(self):
        # RealDiceRandomSource instances provide a choice method
        src = RealDiceRandomSource(None)
        assert hasattr(src, 'choice')

    def test_registered_as_realdice(self):
        # The RealDiceRandomSource is registered as entry point with
        # name 'realdice' in group 'diceware_random_sources'
        sources_dict = dict()
        for entry_point in pkg_resources.iter_entry_points(
                group="diceware_random_sources"):
            sources_dict.update({entry_point.name: entry_point.load()})
        assert 'realdice' in sources_dict
        assert sources_dict['realdice'] == RealDiceRandomSource

    def test_choice_accepts_lists_of_numbers(self, monkeypatch):
        # the choice() method accepts lists of numbers
        self.fake_input_values(["1"], monkeypatch)
        src = RealDiceRandomSource(None)
        assert src.choice([11, 12, 13, 14, 15, 16]) == 11

    def test_choice_accepts_tuples_of_numbers(self, monkeypatch):
        # the choice() method accepts tuples of numbers
        self.fake_input_values(["1"], monkeypatch)
        src = RealDiceRandomSource(None)
        assert src.choice((11, 12, 13, 14, 15, 16), ) == 11

    def test_choice_accepts_list_of_chars(self, monkeypatch):
        # the choice() method accepts lists of chars
        self.fake_input_values(["1"], monkeypatch)
        src = RealDiceRandomSource(None)
        assert src.choice(['a', 'b', 'c', 'd', 'e', 'f']) == 'a'

    def test_choice_accepts_list_of_strings(self, monkeypatch):
        # the choice() method accepts lists of strings
        self.fake_input_values(["1"], monkeypatch)
        src = RealDiceRandomSource(None)
        assert src.choice(
            ['val1', 'val2', 'val3', 'val4', 'val5', 'val6']) == "val1"

    def test_choice_num_of_dice_for_seq_len36(self, monkeypatch):
        # choice() requires two dice for a sequence len of 6**2
        self.fake_input_values(["1", "2"], monkeypatch)
        src = RealDiceRandomSource(None)
        sequence = list(range(6 ** 2))
        expected_index = 6 * (1 - 1) + (2 - 1)     # = 6 x roll_1 + roll_2 - 1
        assert src.choice(sequence) == sequence[expected_index]

    def test_choice_num_of_dice_for_seq_len216(self, monkeypatch):
        # choice() requires three dice for a sequence len of 6**3
        self.fake_input_values(["1", "2", "3"], monkeypatch)
        src = RealDiceRandomSource(None)
        sequence = list(range(6 ** 3))        # 216
        # = 6^2 * (roll_1 - 1) + 6^1 * (roll_2 - 1) + (roll_3 - 1)
        expected_index = 0 + 6 + 3 - 1
        assert src.choice(sequence) == sequence[expected_index]

    def test_hint_if_entropy_is_decreased(self, monkeypatch, capsys):
        # if len of choice is not a multiple of 6, entropy is decreased
        # (not the whole sequence is taken into consideration). We get
        # a warning in that case.
        self.fake_input_values(["1"], monkeypatch)
        src = RealDiceRandomSource(None)
        picked = src.choice([1, 2, 3, 4, 5, 6, 7])
        assert picked == 1
        out, err = capsys.readouterr()
        assert "entropy is reduced" in out
        assert "Using only first 6 of 7 words" in out
        assert err == ""

    def test_no_hint_if_entropy_is_not_decreased(self, monkeypatch, capsys):
        # we do not issue the entropy warning if not neccessary
        self.fake_input_values(["1"] * 6, monkeypatch)
        src = RealDiceRandomSource(None)
        picked1 = src.choice([1, 2, 3, 4, 5, 6])
        picked2 = src.choice(range(1, 6 ** 2 + 1))
        picked3 = src.choice(range(1, 6 ** 3 + 1))
        assert picked1 == 1
        assert picked2 == 1
        assert picked3 == 1
        out, err = capsys.readouterr()
        assert "entropy is reduced" not in out
        assert err == ""

    def test_non_numbers_as_input_are_rejected(self, monkeypatch):
        # Users might input non-numbers. We ask again then.
        self.fake_input_values(["no-number", "", "1"], monkeypatch)
        src = RealDiceRandomSource(None)
        assert src.choice([1, 2, 3, 4, 5, 6]) == 1

    def test_choice_input_lower_value_borders(self, monkeypatch):
        # choice() does not accept "0" but it accepts "1"
        self.fake_input_values(["0", "1"], monkeypatch)
        src = RealDiceRandomSource(None)
        sequence = (1, 2, 3, 4, 5, 6)
        assert src.choice(sequence) == 1

    def test_choice_input_upper_value_borders(self, monkeypatch):
        # choice() does not accept "7" but it accepts "6"
        self.fake_input_values(["7", "6"], monkeypatch)
        src = RealDiceRandomSource(None)
        sequence = (1, 2, 3, 4, 5, 6)
        assert src.choice(sequence) == 6

    def test_pre_check_no_rolls_cause_exception(self):
        # we cannot pick zero items of a sequence
        src = RealDiceRandomSource(None)
        with pytest.raises(ValueError):
            src.pre_check(0, list(range(6)))

    def test_pre_check_warn_if_not_all_seq_items_used(self, capsys):
        # we issue a warning if not all sequence items will be used
        src = RealDiceRandomSource(None)
        src.dice_sides = 10
        src.pre_check(1, list(range(10)))
        out, err = capsys.readouterr()
        assert "entropy is reduced" not in out
        src.pre_check(1, list(range(11)))
        out, err = capsys.readouterr()
        assert "entropy is reduced" in out

    def test_pre_check_requests_rolling_dice(self, capsys):
        # we request the user to roll dice
        src = RealDiceRandomSource(None)
        src.pre_check(5, ['doesntmatter'])
        out, err = capsys.readouterr()
        assert "Please roll 5 dice (or a single dice 5 times)." in out

    def test_sequence_less_than_dice_sides(self, capsys, monkeypatch):
        # Test to see whether we can use a n-sided die to choose from
        # a sequence with less than n items
        src = RealDiceRandomSource(None)
        src.dice_sides = 6
        # A length of 1 requires no rolls
        self.fake_input_values(["1"], monkeypatch)
        picked = src.choice([1])
        out, err = capsys.readouterr()
        assert "roll" not in out
        assert picked == 1
        # A length of 2,3 only requires 1 roll
        for choice_length in (2, 3):
            self.fake_input_values(["1"], monkeypatch)
            picked = src.choice(range(1, choice_length + 1))
            out, err = capsys.readouterr()
            assert "roll 1 dice" in out
            assert picked == 1
        # A length of 4,5 requires 2 rolls
        for choice_length in (4, 5):
            self.fake_input_values(["1", "1"], monkeypatch)
            picked = src.choice(range(1, choice_length + 1))
            out, err = capsys.readouterr()
            assert "roll 2 dice" in out
            assert picked == 1

    def test_dice_sides_respected(self, capsys, monkeypatch):
        # we use the number of dice sides given by options dict.
        self.fake_input_values(["1", "2"], monkeypatch)
        # A Namespace, not a dict, is passed to the constructor.
        options = argparse.Namespace(dice_sides=2)  # a coin
        src = RealDiceRandomSource(options)
        picked = src.choice(['a', 'b', 'c', 'd'])
        out, err = capsys.readouterr()
        # must throw a coin 2 times to pick one out of 4 items
        assert "Please roll 2 dice" in out
        assert picked == 'b'

    def test_main_with_realdice_source(
            self, argv_handler, capsys, monkeypatch):
        # we can run main with `realdice` source of randomness
        self.fake_input_values(
            ["1", "3"], monkeypatch)
        sys.stdin = StringIO("w1\nw2\nw3\nw4\nw5\nw6\n")
        sys.argv = ['diceware', '-r', 'realdice', '-n', '2', '-d', '#', '-']
        main()
        out, err = capsys.readouterr()
        assert out.endswith('W1#W3\n')



import os
from setuptools import setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup_requires = [
    'pytest_runner',
    ]

install_requires = [
    'setuptools',
    ]

tests_require = [
    # See tox.ini
    'pytest >=2.8.3',
    'coverage',
    ]

docs_require = [
    'Sphinx',
    'sphinx_rtd_theme',
    ]

setup(
    name="diceware",
    version="0.9.1",
    author="Uli Fouquet",
    author_email="uli@gnufix.de",
    description=(
        "Passphrases you will remember."),
    license="GPL 3.0",
    keywords="diceware password passphrase",
    url="https://github.com/ulif/diceware/",
    py_modules=[],
    packages=['diceware', ],
    namespace_packages=[],
    long_description=read('README.rst') + '\n\n\n' + read('CHANGES.rst'),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: System Administrators",
        "Topic :: Utilities",
        "Topic :: Security :: Cryptography",
        (
            "License :: OSI Approved :: "
            "GNU General Public License v3 or later (GPLv3+)"),
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    include_package_data=True,
    zip_safe=False,
    setup_requires=setup_requires,
    install_requires=install_requires,
    tests_require=tests_require,
    extras_require=dict(
        tests=tests_require,
        docs=docs_require,
        ),
    entry_points={
        'console_scripts': [
            'diceware = diceware:main',
        ],
        'diceware_random_sources': [
            'system = diceware.random_sources:SystemRandomSource',
            'realdice = diceware.random_sources:RealDiceRandomSource',
            # add more sources of randomness here...
        ],
    },
)



#  diceware -- passphrases to remember
#  Copyright (C) 2015, 2016  Uli Fouquet
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""config -- diceware configuration

`diceware` is configurable via commandline, configuration files and
direct API calls.

"""
try:
    import configparser                  # Python 3.x
except ImportError:                      # pragma: no cover
    import ConfigParser as configparser  # Python 2.x
import os


OPTIONS_DEFAULTS = dict(
    num=6,
    caps=True,
    specials=0,
    delimiter="",
    randomsource="system",
    verbose=0,
    wordlist="en_securedrop",
    dice_sides=6,
    )


def valid_locations():
    """The list of valid paths we look up for config files.
    """
    user_home = os.path.expanduser("~")
    result = []
    if user_home != "~":
        result = [os.path.join(user_home, ".diceware.ini"), ]
    return result


def get_configparser(path_list=None):
    """Parse `path_list` for config values.

    If no list is given we use `valid_locations()`.

    Return a list of paths read and a config parser instance.
    """
    if path_list is None:
        path_list = valid_locations()
    parser = configparser.SafeConfigParser()
    found = parser.read(path_list)
    return found, parser


def get_config_dict(
        path_list=None, defaults_dict=OPTIONS_DEFAULTS, section="diceware"):
    """Get config values found in files from `path_list`.

    Read files in `path_list` config files and return option values from
    section `section` as regular dictonary.

    We only accept values for which a default exists in
    `defaults_dict`. If `defaults_dict` is ``None`` we use
    ``OPTIONS_DEFAULTS``.

    Values are interpolated to have same value type as same-named values
    from `defaults_dict` if they are integers or boolean.

    String/text values are stripped from preceding/trailing quotes
    (single and double).
    """
    result = dict(defaults_dict)
    found, parser = get_configparser(path_list)
    for key, val in defaults_dict.items():
        if not parser.has_option(section, key):
            continue
        if isinstance(val, bool):
            result[key] = parser.getboolean(section, key)
        elif isinstance(val, int):
            result[key] = parser.getint(section, key)
        else:
            result[key] = parser.get(section, key).strip("\"'")
    return result
